local L = AceLibrary("AceLocale-2.2"):new("SellValues")
SellValues = KC_Items:NewModule("SellValues", "AceEvent-2.0", "AceHook-2.1")

function SellValues:OnInitialize()
    self.db = KC_Items:AcquireDBNamespace("SellValues")
    
    if type(self.db.account.data) == "string" then self.db.account.data = {} end
    
    KC_Items:RegisterDefaults("SellValues", 'account', {
        data = {
            ['*'] = "",
        },
    })
    
    KC_Items:RegisterDefaults("SellValues", 'profile', {
        labels = {
            sv = "|cffffff78Vendor pays|r",
            bv = "|cffffff78Vendor charges|r",
            sep = " ",
        },
    })

    KC_Items.options.args.sellvalues = {
	    name = L["Sell Values"],
	    desc = L["Toggle the Sell Values module"],
	    type = "toggle",
	    get = function() return KC_Items:IsModuleActive("SellValues") end,
	    set = function(v) KC_Items:ToggleModuleActive("SellValues", v) end
    }
end

function SellValues:OnEnable()
	self:RegisterEvent("MERCHANT_SHOW")
    self:RegisterEvent("KC_Items_ItemTooltipShown", "ItemTooltipShown")
end

function SellValues:ItemTooltipShown(tt)
    local sv, bv = self:GetPrices(tt.link)
    
    tt:Prep()
    
    if sv and sv > 0 then tt:AddPriceLine(self:Round(sv * tt.qty), self.db.profile.labels.sv, self.db.profile.labels.sep) end
    if bv and bv > 0 then tt:AddPriceLine(self:Round(bv * tt.qty), self.db.profile.labels.bv, self.db.profile.labels.sep) end
    
    tt:CleanUp()
end

function SellValues:MERCHANT_SHOW()

    self:Hook("SetTooltipMoney",true)
	self._itemPrice = nil

	for bag = 0, 4 do
		for slot = 1, GetContainerNumSlots(bag) do
			GameTooltip:SetBagItem(bag, slot)
			if(self._itemPrice) then
				local _, qty = GetContainerItemInfo(bag, slot)
				self:SetPrices(GetContainerItemLink(bag, slot), self._itemPrice/qty, nil)
				self._itemPrice = nil
			end
		end
	end
	self:Unhook("SetTooltipMoney")
    
    for item = 1, GetMerchantNumItems() do
		local _, _, price, qty = GetMerchantItemInfo(item)
		self:SetPrices(GetMerchantItemLink(item), nil, price/qty)
	end
end

function SellValues:SetTooltipMoney(obj, money)
	self._itemPrice = money
    self.hooks.SetTooltipMoney(obj, money)
end

function SellValues:SetPrices(link, sellvalue, buyvalue)
    if not link then return end
    if sellvalue then sellvalue = self:Round(sellvalue) end
    if buyvalue then buyvalue = self:Round(buyvalue) end
    local code = self:GetShortCode(link)
    local data = self.db.account.data[math.fmod(tonumber(code), 50)]
    local sv, bv = data:match(code..",(%d+%.?%d*),(%d+%.?%d*);")
    if sv then
        data = data:gsub(code..",(%d+%.?%d*),(%d+%.?%d*);", ("%s,%s,%s;"):format(code, sellvalue or sv, buyvalue or bv))
    else
        data= ("%s%s,%s,%s;"):format(data, code, sellvalue or 0, buyvalue or 0)
    end
    self.db.account.data[math.fmod(tonumber(code), 50)] = data
end

function SellValues:GetPrices(link)
    if not link then return end
    local code = self:GetShortCode(link)
    local data = self.db.account.data[math.fmod(tonumber(code), 50)]
    local sv, bv = data:match(code..",(%d+%.?%d*),(%d+%.?%d*);")
    return tonumber(sv), tonumber(bv)
end
